import scrapy
import re
import logging
from crawl.comm.basketball import BSTResult
from crawl.spiders.util import getNoneStr


class BasketballSpider(scrapy.Spider):
  name = "basketball-result"

  # 所有比赛
  _matchesMap = {}

  def start_requests(self):
    url = 'https://info.sporttery.cn/basketball/match_result.php'
    yield scrapy.Request(url, self.parseResult, 'GET')

  def parseResult(self, response):
    # 处理比赛结果
    cssOfMatch = '.all-wrap > .match_list .m-tab tr'
    matches = response.css(cssOfMatch)
    if matches is None: return
    for matchNode in matches:
      tdNodeList = matchNode.css('td')
      if tdNodeList is None or len(tdNodeList) < 10:
        continue

      matchTime = getNoneStr(tdNodeList[0].css('::text').get())
      matchWeek = getNoneStr(tdNodeList[1].css('::text').get())
      league = getNoneStr(tdNodeList[2].css('::text').get())
      leagueFullName = getNoneStr(tdNodeList[2].attrib.get('title'))  # 联赛全称
      leagueName = '|'.join((league, leagueFullName))
      homeTeam = self.trimBrackets(getNoneStr(tdNodeList[3].css('.zhu::text').get()))  # 主队
      awayTeam = self.trimBrackets(getNoneStr(tdNodeList[3].css('.ke::text').get()))  # 客队

      half = ''
      halfSpan = tdNodeList[4].css('span')
      if halfSpan is not None and len(halfSpan) > 0:
        part1 = getNoneStr(tdNodeList[4].css("span::text")[0].get())
        part2 = getNoneStr(tdNodeList[4].css("span::text")[1].get())
        part3 = getNoneStr(tdNodeList[5].css("span::text")[0].get())
        part4 = getNoneStr(tdNodeList[5].css("span::text")[1].get())
        part5 = getNoneStr(tdNodeList[6].css('::text').get())
        half = '|'.join((part1, part2, part3, part4, part5))

      whole = getNoneStr(tdNodeList[7].css('span::text').get())
      status = getNoneStr(tdNodeList[11].css('::text').get())

      if whole == '无效场次':
        status = '无效场次'

      bstResult = BSTResult(
        matchTime,
        matchWeek,
        leagueName,
        homeTeam,
        awayTeam,
        half,
        whole,
        status,
        ''
      )

      matchLink = getNoneStr(tdNodeList[3].css('a').attrib.get('href'))
      matchId = re.sub(r'^.*bk_match_info.php\?m=', '', matchLink)

      # 加入到临时字典里
      self._matchesMap[matchId] = bstResult

      if status == '已完成':
        link = tdNodeList[12].css('a').attrib.get('href')
        yield scrapy.Request('https:' + link, self.parsePrice, 'GET')
      else:
        logging.info("采集到数据 --> %s" % bstResult.toString())
        bstResult.persist()

    # 是否存在下一页
    pgNodes = response.css('.m-page .u-pg2')
    if pgNodes is None or len(pgNodes) < 1: return

    # 总页数
    total = pgNodes[-1].css('a::text').get()
    if total is None or total == '':
      total = 0

    # 当前页数
    current = re.sub(r'^.*match_result.php[?page=]*', '', response.url)
    if current is None or current == '':
      current = 1

    if int(current) >= int(total): return

    # 下一页
    nextPG = int(current) + 1
    url = 'https://info.sporttery.cn/football/match_result.php?page=' + str(nextPG)
    yield scrapy.Request(url, self.parseResult, 'GET')


  def trimBrackets(self, str):
    if str is None: return ''

    return re.sub(r'\(.*\)', '', str)

  def parsePrice(self, response):
    logging.info("采集数据源 ---> %s" % response.url)

    tableList = response.css('.kj-table')
    if tableList is None or len(tableList) < 4:
      logging.error("抓取玩法结果失败")
      return

    playRes = []

    for i, tab in enumerate(tableList):

      try:
        if i == 0 or i == 3:
          # 胜负 or 胜分差
          tr = tab.css("tr")[0]
          tdList = tr.css('td')
          playRes.append(getNoneStr(tdList[1].css("span::text").get()).replace(' ', ''))
        elif i == 1 or i == 2:
          # 让分胜负 or 大小分
          tr = tab.css("tr")[-1]
          tdList = tr.css('td')
          playRes.append(getNoneStr(tdList[-1].css("span::text").get()).replace(' ', ''))
        else:
          continue
      except:
        continue

    matchId = re.sub(r'^.*pool_result.php\?id=', '', response.url)
    bstResult = self._matchesMap[matchId]
    bstResult.playRes = '|'.join(playRes)

    #
    logging.info("采集到数据 --> %s" % bstResult.toString())

    # 入库
    bstResult.persist()
